##############################################
##Specification of dir, input and output dir##
##############################################
##Set working directory, which contains the directory "Bayesian_PBM_Analysis"
dir <- "/Users/boj/Documents/PBM/"
dir <- paste(dir,"Bayesian_PBM_Analysis/",sep="")

#Set directory that contains input and out directories
dir <- paste(dir,"anova_example/",sep="")

#Set input directory, which contains names.txt, kmers.txt, family.txt and score.txt 
input.dir <- paste(dir,"input/",sep="")

#Set output directory, which contains the output results from running "pbm.anova"
output.dir <- paste(dir,"output/",sep="")

#Set fam index of TF-common k-mers, tf index of TF-preferred k-mers, or TF-preferred by tf1 but not tf2. Assign 'NULL' value to variables if not used.
fam <- 1
tf <- 1
tf1 <- 1
tf2 <- 2

######################################
##Analysis of results from pbm.anova##
######################################
anova.result <- function(dir, input.dir, output.dir, fam=NULL, tf=NULL, tf1=NULL, tf2=NULL)
{
    library(gplots)
    
    setwd(input.dir)
    name <- read.table("names.txt",header=F)
    names <- name$V1
    domain <- name$V2
    kmers <- read.table("kmers.txt",header=F)
    x <- read.table("score.txt",header=F)
    x <- as.matrix(x)
    nK <- 32896
    nTF <- length(names)
    family <- unique(domain)
    nFam <- length(family)
    fIndex <- NULL
    fSize <- NULL
    for(i in 1:nTF) {
        fIndex[i] <- which(family==domain[i])
    }
    for(i in 1:nFam) {
        fSize[i] <- sum(domain==family[i])
    }
    
    setwd(output.dir)
    trace <- read.table("trace.txt",header=T)
    param <- read.table("param.txt",header=T)
    par(mfrow=c(2,2))
    par(mar=c(6,6,4,4))
    plot(trace$logLik,type="l",col=4,ylab="log-Likelihood",xlab="Iteration",main="(a) Trace Plot of log-Likelihood",cex.lab=1.5,cex.axis=1.5,cex.main=1.3)
    abline(v=param$nBurn,lty=3)
    acf(trace$logLik,main="(b) ACF of log-Likelihood",ylab="log-Likelihood",cex.lab=1.5,cex.axis=1.5,cex.main=1.0)
    plot(trace$sigma,type="l",col=2,ylab="Sigma",xlab="Iteration",main="(c) Trace Plot of Sigma",cex.lab=1.5,cex.axis=1.5,cex.main=1.3)
    abline(v=param$nBurn,lty=3)
    acf(trace$sigma,main="(d) ACF of Sigma",ylab="Sigma",cex.lab=1.5,cex.axis=1.5,cex.main=1.0)
    #plot(trace$kappa,type="l",col=4,ylab="gamma",xlab="iteration",main="Trace Plot of Gamma")
    #abline(v=param$nBurn,lty=3)
    #acf(trace$kappa,main="ACF of Gamma")
    
    fbind <- read.table("familybindfreq.txt",header=F)
    tbind <- read.table("tfbindfreq.txt",header=F)
    fprob <- read.table("familybindprob.txt",header=F)
    tprob <- read.table("tfbindprob.txt",header=F)
    fnbind <- read.table("familynobindfreq.txt",header=F)
    tnbind <- read.table("tfnobindfreq.txt",header=F)
    fnprob <- read.table("familynobindprob.txt",header=F)
    tnprob <- read.table("tfnobindprob.txt",header=F)
    tau <- read.table("tau.txt",header=F)
    tau <- tau$V1
    
    setwd(dir)
    #Heatmap for top 50 'sticky' 8-mers
    pdf(file=paste(dir,"top_50_skicky_heatmap.pdf",sep=""))
    id1 <- sort(tau,decreasing=T,index.return=T)$ix[1:50]
    xx <- x[id1,]
    heatmap.2(xx, col=topo.colors(75), scale="none", distfun=dist, dendrogram="none", Rowv=F, Colv=F,
    key=TRUE, symkey=FALSE, density.info="none",trace="none",
    labCol=paste(names,"||",domain),cexCol=0.3, 
    labRow=paste(kmers[id1,1],kmers[id1,2]), cexRow=0.5, margins = c(3, 8))
    dev.off()
    
    #Top 50 'sticky' 8-mer sequences
    seq <- as.matrix(kmers[id1,1])
    write(seq,file="sticky_kmers.seq")
    
    if(length(fam) != 0)
    {
        #Heatmap for TF-common 8-mers for TF family fam
        pdf(file=paste(dir,"tf_common_heatmap.pdf",sep=""))
        id <- seq(1:32896)
        id1 <- id[fbind[,fam]>0.5]
        id1 <- sort(fbind[,fam],decreasing=T,index.return=T)$ix[1:length(id1)]
        xx <- x[id1,]
        heatmap.2(xx, col=topo.colors(75), scale="none", distfun=dist, dendrogram="none", Rowv=F, Colv=F,
        key=TRUE, symkey=FALSE, density.info="none",trace="none",
        labCol=paste(names,"||",domain),cexCol=0.3, 
        labRow=paste(kmers[id1,1],kmers[id1,2]), cexRow=0.3, margins = c(3, 8))
        dev.off()
        
        #A different version of heatmap
        #library(made4)
        #heatplot(xx, dend="row", cols.default=T, margin=c(8,1),labCol=paste(names,"||",domain), labRow=FALSE,lowcol="green", highcol="red", scale="none",classvec=domain)
        
        setwd(dir)
        #TF-common 8-mer sequences
        seq <- as.matrix(kmers[id1,1])
        write(seq,file="tf_common_kmers.seq")
    }
    
    if(length(tf) != 0)
    {
        #Heatmap for TF-preferred 8-mers for tf
        pdf(file=paste(dir,"tf_preferred_heatmap.pdf",sep=""))
        id <- seq(1:32896)
        id1 <- id[tbind[,tf]>0.5]
        id1 <- sort(tbind[,tf],decreasing=T,index.return=T)$ix[1:length(id1)]
        xx <- x[id1,]
        library(gplots)
        heatmap.2(xx, col=topo.colors(75), scale="none", distfun=dist, dendrogram="none", Rowv=F, Colv=F,
        key=TRUE, symkey=FALSE, density.info="none",trace="none",
        labCol=paste(names,"||",domain),cexCol=0.3, 
        labRow=paste(kmers[id1,1],kmers[id1,2]), cexRow=0.5, margins = c(3, 8))
        dev.off()
    }
    
    if(length(tf1) != 0 & length(tf2) != 0)
    {
        pdf(file=paste(dir,"tf_distinct_heatmap.pdf",sep=""))
        #8-mers preferred by tf1 but not tf2
        id <- seq(1:32896)
        id1 <- id[tbind[,tf1]>0.5&tbind[,tf2]<0.01]
        xx <- x[id1,]
        library(gplots)
        heatmap.2(xx, col=topo.colors(75), scale="none", distfun=dist, dendrogram="none", Rowv=F, Colv=F,
        key=TRUE, symkey=FALSE, density.info="none",trace="none",
        labCol=paste(names,"||",domain),cexCol=0.3, 
        labRow=paste(kmers[id1,1],kmers[id1,2]), cexRow=0.8, margins = c(3, 10))
        dev.off()
    }
}

anova.result(dir, input.dir, output.dir, fam, tf, tf1, tf2)

