######################################################
##Specification of dir, input files and output files##
######################################################
##Set working directory, which contains the directory "Bayesian_PBM_Analysis"
dir <- "/Users/boj/Documents/PBM/"
dir <- paste(dir,"Bayesian_PBM_Analysis/",sep="")

#Set input directory, which contains estimated k-mer background noise file and 8-mer observed E-score files from PBM experiments
dir <- paste(dir,"background_example/",sep="")

#Estimated background noise file
noise.file <- "estimated_noise.txt"

#PBM 8-mer observed E-score file, which havs the first four columns as: "8-mer, 8-mer(reverse complementary), median intensity and observed E-score" and separated by tab
input.file <- "Rpn4-9_8mers_11111111.txt"

#Output file for corrected E-scores, format is the same as the input file with observed E-scores substituted by corrected E-scores
output.file <- "Rpn4-9_8mers_corrected.txt"

#Threshold to select specific binding k-mers
threshold <- 0.35

#####################################
##k-mer background noise correction##
#####################################
background.correction <- function(dir, input.file, output.file, noise.file, threshold=0.35)
{
    setwd(dir)
    tau <- read.table(noise.file,header=F)
    tau <- tau$V1
    m=-0.0196628	
    s=0.151863

    #Read 8-mer observed E-score
    data <- read.table(input.file,header=F)
    x <- data$V4
    y <- x-tau*s
    xx <- rep(0,length(x))
    id <- sort.int(y,index.return=T,decreasing=T)$ix
    yy <- rep(0, length(y))
    if(sum(x>=threshold)>0)
    {
        id <- id[1:sum(x>=threshold)]
        yy[id] <- 1
    }
    xx <- yy

    #Output corrected E-scores to outputfile
    data1<-data
    data1$V4 <- y
    write.table(data1,file=output.file,row.names = F, col.names = T, quote=F, sep="\t")

    pdf(file=paste(dir,"background_noise_hist.pdf",sep=""))
    hist(s*tau,xlab="Estimated k-mer background noise",main="",breaks=100)
    dev.off()
    pdf(file=paste(dir,"background_noise_plot.pdf",sep=""))
    plot(x,s*tau,xlab="Observed E-score",ylab="Noise",pch=20,cex=0.5,cex.axis=1.5,cex.lab=1.5)
    points(x[xx>0],s*tau[xx>0],col=2,pch=20)
    abline(v=threshold,col=4,lty=2,lwd=2)
    dev.off()

    #8-mer sequences with observed E-score >= threshold
    id1 <- seq(1:32896)
    id1 <- id1[x>=threshold]
    seq <- as.matrix(data$V1[id1])
    write(seq,file="observed_kmers.seq")
    
    #8-mer sequences with corrected E-score
    id1<-id
    seq <- as.matrix(data$V1[id1])
    write(seq,file="corrected_kmers.seq")
}

background.correction(dir,input.file,output.file,noise.file, threshold)


