/*
 g++ -o pbm.partition pbm.partition.cpp lgamma.cpp random.cpp
 */

#include <iostream>
#include <cstring>
#include <cmath>
#include <cstdlib>
#include <ctime>
#include <sys/time.h>
using namespace std;

#include "random.h"
#include "lgamma.h"

#define LINESIZE 1024 // command-line size
#define MAX_T 1 //maximum number of temperature ladders
#define MAX_D 100 //maximum number of moduels
#define MAX_G 100 //maximum number of responses group
#define MAX_V 4	//maximum number of covariate values (discrete)

#define MAX_CL 20 //maximum number of clusters
#define INIT_CL 3 //initial number of clusters
#define MAX_GX 1 //maximum number of covariates in a group
#define MAX_GY 1000 //maximum number of responses in a group
#define MAX_X 1 //maximum number of covariate groups in a moodule
#define MAX_Y 20 //maximum number of response groups in a module
#define MAX_B 1 //maximum block size

#define square(x) (double)((double)(x)*(x))

static KW_RNG::RNG _my_random;
void my_srandom(int x) { _my_random.init(x,!x*13,x*x+1,(x>>16)+(x<<16)); }
double my_random01() { return _my_random.rand_closed01(); }
double my_uniform(double x) {return _my_random.uniform(0.0,x); }
int my_binomial(double pp, int n) { return _my_random.binomial(pp,n); }
double my_normal(double mu = 0.0, double sd = 1.0) { return _my_random.normal(mu,sd); }
double my_chisq(double df) { return _my_random.chi_square(df); }

#ifdef _WIN32
#include <process.h>
#include <windows.h>
void set_priority_low() {
  HANDLE hProcess=OpenProcess(PROCESS_ALL_ACCESS,TRUE,_getpid());
  SetPriorityClass(hProcess,IDLE_PRIORITY_CLASS);
}
#else
#include <unistd.h>
#endif

/**********************************
*
*Data type definition
*
***********************************/
struct GroupY {
	unsigned int nY, nCL; // number of covariates and clusters 
	double kappa, zeta; // hyper-parameters
	double *sumY; //sum of response for each individual, dimN
	double ssqY; // sum of squares for responses in the group

	unsigned int *sI; // sample cluster indicator, dimN
	unsigned int *cID;// cluster ID, MAX_CL
	unsigned int *cN; // cluster size, MAX_CL
	double *sumC; // sum of response in each cluster, MAX_CL
    double ssqC; // sum of squares for responses in clusters
    
	double logLikelihood; // log-likelihood for responses in a group
    double logDirPrior; // log prior of dirichlet process
    unsigned int moduleID; // module ID: 0 for NULL
};

struct GroupX {
	unsigned int nX; // number of covariates
	unsigned int *code; // covariate code for each sample, dimN
	int *xID;// covariate ID, MAX_GX 
    int *pos; // position of covariate in code, MAX_GX 
	
	unsigned int **pattern; // **_pattern; // covariate patterns in each cluster, MAX_CL x dimN
	unsigned int **count; // **_count; // counts of covariates patterns, MAX_CL x dimN
	unsigned int *cK; // number of non-zero counts in each cluster, MAX_CL
	
	double logLikelihood; // log-likelihood for covariates in a group
    unsigned int moduleID; // module ID: 0 for NULL
};

struct Module {
	double chisi; // temprature
	double logLik, logLikX, logLikY, logDirPrior; // log-likelihood for all, covariates, responses, cluster priors
	unsigned int nYG, nXG, nX, nY, nCL; // number of response groups, number of covariate groups, number of covariates, number of responses, number of clusters TODO: do we need nX and nY?
	
	unsigned int *sI; // sample cluster indicator, dimN
	unsigned int *cID;// cluster ID, MAX_CL
	unsigned int *cN; // cluster size, MAX_CL
	
	struct GroupX *groupX; // groups of covariates, MAX_X
	unsigned int *groupXID; // location of groups, MAX_X
	unsigned int *groupYID; // location of groups, MAX_Y
};

struct Record {
	double chisi;
	double logLik;
	double logLikNullY;
	double logLikNullX;
    double logLikB;
	unsigned int *xI; // module indicator
	unsigned int *xG; // group indicator
	unsigned int *yI; // module indicator of groups
	unsigned int *yG; //group indicator
};


/***********************************
*
*Global variables
*
**********************************/
char inputDir[LINESIZE];
char outputDir[LINESIZE];
char fileName[LINESIZE];

unsigned int dimN, dimP, dimQ, dimF; // sample size, number of covariates,  number of responses
unsigned int nD, nG, nT, nV; // number of module, reponse groups, temparature ladders, and covariate values
unsigned int nInit, nBurn, nGap; // number of iterations for initialization, sampling and burning, number of steps between heavy computations
double betaX = 0.01, betaY = 0.01, betaB = 0.01, betaGY = 0.01, betaC = 0.01; // priors for covariates in groups, responses in modules, blocks, clusters, responses in group, response clusters
double lambdaB = 0.2, lambda = 1.0, nu = 1.0, tau = sqrt(1.0); // pseudo-count, variance priors sqrt(0.5) sqrt(2.0)
double kappa = 1.0, zeta = 1.0; // scale priors 0.5 2.0

double *Y; // responses
unsigned int *X; // covariates
double *chisiT; // temperatures
double *PowerX, *PowerV; // power array
double *LgammaY, *LgammaC; // log-gamma vector for responses
double **LgammaX, **LgammaB; // log-gamma vector for covariates and blocks

double *logLikSingleX, *logLikSingleY; // single covariate/response likelihood
double logLikNullX, logLikNullY; // background covariates/responses likelihood
    
double *logLikEndX; // block end probability (logged)
double **logLikBlockX; // probability of forming block from i to i+b, 0 <= b <= MAX_B
int *blockBound; // block start indicator: 1 chromsome; 2 LD block

unsigned int **_pattern, **_count; // pattern and count of covariates
unsigned int *_code, *_patternN; // individual codes and number of patterns

double **resultGY; // result for response group
double **resultGX; // result for covariate group
double **resultX; // result for module covariate
double **resultY; // result for module response
int **resultC; // result for cluster
int **resultCY; // result for response cluster
double *resultB; // result for block
double *logLikInit; // logLikelihood trace
double *logLikBurn; // logLikelihood trace
double *logLikIter; // logLikelihood trace

//unsigned int* familyId;
//unsigned int* familySize;
double meanAll, sdAll;

/**********************************
*
*Functions
*
************************************/
/* Basic Functions */
void errorExit( const char *msg );
double diffClock(clock_t clock1,clock_t clock2);
void hpsort(unsigned long n, double *ra, int *index);
void unifSample( unsigned int N, unsigned int n, unsigned int *value );
unsigned int multiSample( double* pp, unsigned int n, unsigned int *indicator = NULL, double chisi = 1.0 );

/* Input Processing */
void readResponse( );

/* Initialization */
void initializePower( );
void initializeLgamma( double lambda, double nu );
double calcSingleY( );

double calcSingleX( );
void calcBlock( );
void initBlock( double betaB );

void initializeGroupY( struct GroupY *groupY, struct Record *record, double betaC );
void cleanUpGroupY( struct GroupY *groupY, struct Record *record );

void initializeModules( struct GroupY* groupY, struct Module *module, struct Record* record ); 
void cleanUpModules( struct Module *module );

/* Main Update */
double updateBlock( int output = 1 );
void updateGroupY( struct GroupY* groupY, struct Record *record, double betaGY, int output = 1 );
void updateClusterY( struct GroupY* groupY, double betaC );

void updateCluster( struct Module* module, struct GroupY *response, double betaC );
void updateResponse( struct GroupY *groupY, struct Module* module, struct Record *record, double betaY, double betaGY, int output = 1 );

/* Output */
void writeResultModules( double **resultX, double **resultY, double **resultGX, double **resultGY, int **resultC, double *resultB );

void writeResultGroupY( double **resultGY, int **resultCY );

void printUsage()
{
	fprintf(stderr, "pbm.partition usage:\n\n");
	fprintf(stderr, "\t-i <input file>\n");
	fprintf(stderr, "\t-o <output directory>\n");
    fprintf(stderr, "\t-n <total number of TFs>\n");
	fprintf(stderr, "\t-m <total number of k-mer groups, default: 100>\n");
	fprintf(stderr, "\t-t <number of burn-in iterations, default: 100>\n");
	fprintf(stderr, "\t-s <number of mcmc sampling iterations, default: 1000>\n");
	exit(0);
}

void parseInput(int argc, char *argv[])
{
	extern char *optarg;
	float tempFloat;
	int opt, tempInt;
    
	while ((opt = getopt(argc, argv, "i:o:n:m:t:s:"))!=EOF)
	{
		switch(opt)
		{
			case 'i':
				sscanf(optarg, "%s", inputDir);
				break;
			case 'o':
				sscanf(optarg, "%s", outputDir);
				break;
            case 'n':
				if(sscanf(optarg, "%d", &tempInt) && tempInt > 0)
					dimN = tempInt;
				break;
			case 'm':
				if(sscanf(optarg, "%d", &tempInt) && tempInt > 0)
					nG = tempInt;
				break;
			case 't':
				if(sscanf(optarg, "%d", &tempInt) && tempInt > 0)
					nInit = tempInt;
				break;
			case 's':
				if(sscanf(optarg, "%d", &tempInt) && tempInt > 0)
					nBurn = tempInt;
				break;
        }
	}	
}

/************************************
 *
 *main
 *
 *************************************/
int main(int argc, char *argv[]) {
    
#ifdef _WIN32
	clock_t __clock = clock();
	time_t __time;
	set_priority_low();
	my_srandom(__clock + time(&__time) + _getpid());
#else
	my_srandom(getpid());
#endif
    
    if (argc == 1)
		printUsage();
    
	unsigned int i, j, k, g, t, d;
	nInit = 100; 
	nBurn = 1000;
    nGap = 10;
	nT = 1;
	nD = 1;
    nG = 100;
    if(nG < nD)
        nG = nD;
	nV = 1;
	dimP = 0;
	dimQ = 32896;    
	dimN = 173; //Homeo: 173; Failed: 68; zf-C2H2: 43; HMG: 23; ETS: 22; bHLH: 21
    
	parseInput(argc, argv);
    
    Y = (double *) calloc(dimQ*dimN, sizeof(double));
	logLikSingleY = (double *) calloc(dimQ, sizeof(double));
    
    resultGY = (double **) calloc(dimQ, sizeof(double*));
    resultY = (double **) calloc(nG, sizeof(double*));
    resultCY = (int **) calloc(nInit, sizeof(int*));
    resultC = (int **) calloc(nBurn, sizeof(int*));
    for(j = 0; j < dimQ; j++)
    {
        resultGY[j] = (double *) calloc(nG+1, sizeof(double));
        for(g = 0; g < nG+1; g++)
        {
            resultGY[j][g] = 0.0;
        }
    }
    
    for(j = 0; j < nG; j++)
    {
        resultY[j] = (double *) calloc(nD+1, sizeof(double));
        for(g = 0; g < nD+1; g++)
        {
            resultY[j][g] = 0.0;
        }
    }
    for(t = 0; t < nInit; t++)
    {
        resultCY[t] = (int *) calloc(dimN, sizeof(int));
        for(i = 0; i < dimN; i++)
        {
            resultCY[t][i] = 0;
        }
    }
    for(t = 0; t < nBurn; t++)
    {
        resultC[t] = (int *) calloc(dimN, sizeof(int));
        for(i = 0; i < dimN; i++)
        {
            resultC[t][i] = 0;
        }
    }
    
    readResponse( );
    initializePower( ); //Initialize Power
	initializeLgamma( lambda, nu ); //Initialize Lgamma
    
    logLikNullX = 0.0;
	logLikNullY = calcSingleY( ); //Calculate log-likelihood for each response
    
    struct GroupY *groupY = NULL, *tempY;
    struct Record *record = NULL, *tempR;
	if (!(groupY = (struct GroupY *) calloc(nT*nG, sizeof(struct GroupY))))
        errorExit("Can not allocate space for groupY\n");
	if (!(record = (struct Record *) calloc(nT, sizeof(Record))))
        errorExit("Can not allocate space for record\n");
    
    logLikInit = (double *) calloc(nInit, sizeof(double));
    for(t = 0; t < nInit; t++)
        logLikInit[t] = 0.0;
    
    initializeGroupY(groupY, record, betaC);
    clock_t begin, end;
	begin = clock();
    for(t = 0; t < nInit; t++) {
		tempR = record;
        updateGroupY(groupY, tempR, betaGY);
        tempY = groupY;
        for( g = 0; g < nD; g++, tempY++) {
			updateClusterY(tempY, betaC);
            for(i = 0; i < dimN; i++)
                resultCY[t][i] = tempY->sI[i];
        }        
        tempR->logLik = tempR->logLikNullX+tempR->logLikNullY;
        tempY = groupY;
        for( g = 0; g < nG; g++, tempY++) {
            tempR->logLik += tempY->logLikelihood; 
            tempR->logLik += tempY->logDirPrior;
        }
        logLikInit[t] = tempR->logLik;
        cout << "Iteration: " << t << endl;
	}
    end = clock();
	cout << "Total time: " << diffClock(end,begin) << "ms" << endl;
	cout << "Time per run: " << diffClock(end,begin)/nInit << "ms" << endl;
    double initTime = diffClock(end,begin)/nInit;
    
    writeResultGroupY(resultGY, resultCY);
    //FILE *lFile = fopen ("logLikInit.txt" , "w");
    //for(t = 0; t < nInit; t++)
    //    fprintf(lFile,"%f\n",logLikInit[t]);
    //fclose(lFile);
    
    for(j = 0; j < dimQ; j++)
    {
        for(g = 0; g < nG+1; g++)
        {
            resultGY[j][g] = 0.0;
        }
    }
    struct Module *module = NULL, *tempM;
    if (!(module = (struct Module *) calloc(nD, sizeof(struct Module))))
        errorExit("Can not allocate space for module\n");
    
    initializeModules(groupY, module, record);
    logLikBurn = (double *) calloc(nBurn, sizeof(double));
    for(t = 0; t < nBurn; t++)
        logLikBurn[t] = 0.0;
    begin = clock();
    for(t = 0; t < nBurn; t++) {
        tempR = record;
        updateResponse(groupY, module, tempR, betaY, betaGY);

        tempM = module;
        for( d = 0; d < nD; d++, tempM++) {
            //cout << "Module: " << d+1 << endl;
			updateCluster(tempM, groupY, betaC);
            for(i = 0; i < dimN; i++)
                resultC[t][i] = tempM->sI[i];
		}

        if(t%nGap == 0)
        {
            cout << "Update GroupY..." << endl;
            updateGroupY(groupY, tempR, betaGY);
            
            tempM = module;
            for(d = 0; d < nD; d++, tempM++)
            {
                tempM->logLikY = tempM->nYG*log(betaY);
                for(g = 0; g < tempM->nYG; g++)
                {
                    tempY = groupY + tempM->groupYID[g]-1;
                    tempM->logLikY += tempY->logLikelihood;
                }
                tempM->logLik = tempM->logLikX+tempM->logLikY+tempM->logDirPrior;
            }
        }
        tempR->logLik = tempR->logLikNullX+tempR->logLikNullY;
        tempM = module;
        for( d = 0; d < nD; d++, tempM++) {
            tempR->logLik += tempM->logLik;
		}
        tempY = groupY;
        for( g = 0; g < nG; g++, tempY++) {
            if(tempY->moduleID == 0)
            {
                tempR->logLik += tempY->logLikelihood; 
                tempR->logLik += tempY->logDirPrior;
            }
        }
        cout << tempR->logLik << endl;
        logLikBurn[t] = tempR->logLik;
        cout << "Iteration: " << t << endl;
	}
    end = clock();
	cout << "Total time: " << diffClock(end,begin) << "ms" << endl;
	cout << "Time per run: " << diffClock(end,begin)/nBurn << "ms" << endl;
    double burnTime = diffClock(end,begin)/nBurn;
    
    cout << "Time per run (Init): " << initTime << "ms" << endl;
    cout << "Time per run (Burn): " << burnTime << "ms" << endl;
    
    writeResultModules(resultX, resultY, resultGX, resultGY, resultC, resultB);
    strcpy(fileName, outputDir); strcat(fileName,"/trace.txt"); FILE *traceFile = fopen (fileName, "w");
    for(t = 0; t < nBurn; t++)
        fprintf(traceFile,"%f\n",logLikBurn[t]);
    fclose(traceFile);
    cleanUpModules(module);
    
    cleanUpGroupY(groupY, record);
    
    free(groupY);
	free(record);
    //free(module);
    
    free(PowerV);
	free(PowerX);
	for(i = 0; i <= MAX_B+1; i++)
		free(LgammaB[i]);
	for(i = 0; i <= MAX_X; i++)
		free(LgammaX[i]);
    
	free(LgammaB);
	free(LgammaX);
    free(LgammaY);
    free(LgammaC);
	free(logLikSingleY);
	free(Y);
    for(i = 0; i < dimQ; i++)
        free(resultGY[i]);
    free(resultGY);
    for(i = 0; i < nD; i++)
    {
        free(resultCY[i]);
    }
    free(resultCY);
    free(logLikInit);
    
    return 0;
}

/**********************************
*
*errorExit
*
************************************/
void errorExit(const char *msg){
	fprintf(stderr, "%s\n", msg);
	exit(0);
}

/**********************************
*
*diffClock
*
************************************/
double diffClock(clock_t clock1,clock_t clock2)
{
	double diffticks = clock1-clock2;
	double diffms = (diffticks*1000)/CLOCKS_PER_SEC;
	return diffms;
} 

/*****************************************
 *
 *Func: hpsort
 *
 ******************************************/
/*sort an array ra[1..n] into ascending numerical order using the heapsort
 algorithm. n is input. ra is replaced on output by its sorted rearrangement
 index is the returned index*/
void hpsort(unsigned long n, double *ra, int *index)
{
	unsigned long i,ir,j,l;
	double rra;
	int ni, nj;
	if (n < 1) return;
	l=(n >> 1);
	ir=n-1;
	for (;;) {
		if (l > 0) {
			rra=ra[--l];
		} else {
			rra=ra[ir];
			ra[ir]=ra[0];
			ni=index[ir];nj=index[0];
			index[ir]=nj; index[0]=ni;
			if (--ir == 0) {
				ra[0]=rra;
				break;
			}
		}
		i=l;
		j=l+l+1;
		while (j <= ir) {
			if (j < ir && ra[j] < ra[j+1]) j++;
			if (rra < ra[j]) {
				ni=index[i];nj=index[j];
				index[j]=ni;
				index[i]=nj;
				ra[i]=ra[j];
				i=j;
				j=(j <<= 1)+1;
			} else j=ir+1;
		}
		ra[i]=rra;
	}
}

/**********************************
*
*multiSample
*
************************************/
unsigned int multiSample(double* pp, unsigned int n, unsigned int *indicator, double chisi)
{
	double norm = 0.0, r = 0.0, sum = 0.0, max = *(pp);
	unsigned int i = 0;
	for(i = 0; i < n; i++) 
	{
        if(indicator==NULL)
        {
            //cout << i << "\t" << *(pp+i) << endl;
            if(*(pp+i)>max)
                max = *(pp+i);
		} else if(*(indicator+i)==1)
        {
            if(*(pp+i)>max)	
                max = *(pp+i);
        }
	}
    for(i = 0; i < n; i++) 
	{
        if(indicator==NULL)
            norm += exp((*(pp+i)-max)*chisi);
        else if(*(indicator+i)==1)
			norm += exp((*(pp+i)-max)*chisi);
	}
	norm = log( norm );

	r = my_uniform(1.0);
	for(i = 0; i < n; i++) 
	{
        if(indicator==NULL)
        {
			sum += exp((*(pp+i)-max)*chisi - norm);
			if(sum > r)	
				break;
		} else if(*(indicator+i)==1)
		{
			sum += exp((*(pp+i)-max)*chisi - norm);
			if(sum > r)	
				break;
		}
	}
	return i;
}

/**********************************
*
*unifSample
*
************************************/
void unifSample(unsigned int N, unsigned int n, unsigned int *index)
{
	unsigned int i, j;

	if (N < n){
		fprintf(stderr, "unifSample(N, n, *), N must be greater than n\n");
		exit(0);
	}
	i=0;
    do
    {
        index[i] = (unsigned int)floor(my_uniform(1.0)*N);
        j = 0;
        while( index[j] != index[i] && j < i )
            j++;
        if (j == i)
            i++;        
    }
    while (i < n);
}

/**********************************
*
*initializePower
*
************************************/
void initializePower( )
{
	if (!(PowerX = (double *) calloc(MAX_GX+1, sizeof(double))))
		errorExit("cannot allocate space to Power\n");
	PowerX[0] = 1;
	for(unsigned int i = 1; i <= MAX_GX; i++)
		PowerX[i] = PowerX[i-1]*nV;

	if (!(PowerV = (double *) calloc(MAX_GX+1, sizeof(double))))
		errorExit("Cannot allocate space to PowerX\n");
	PowerV[0] = 1;
	for(unsigned int i = 1; i <= MAX_GX; i++)
		PowerV[i] = PowerV[i-1]*MAX_V;

	cout<<"Power Initialized..."<<endl;
}

/**********************************
*
*initializeLgamma

*
************************************/
void initializeLgamma( double lambda, double nu ){
	int ifault;
	double pseudocount;
	unsigned int i, j;
    
	if (!(LgammaX = (double **) calloc(MAX_GX+1, sizeof(double*))))
		errorExit("Cannot allocate space to Lgamma\n");
	for(i = 0; i <= MAX_GX; i++) {
		if (!(LgammaX[i]=(double *) calloc(dimN+1, sizeof(double))))
			errorExit("Can not allocate space to Lgamma\n");
	}
	for(i = 0; i <= MAX_GX; i++) {
		pseudocount = lambda/(double)PowerX[i];
		LgammaX[i][0] = 0;
		for(unsigned int j = 1; j <= dimN; j++)
			LgammaX[i][j] = LgammaX[i][j-1] + log(j-1+pseudocount);
	} 
    
    if (!(LgammaB = (double **) calloc(MAX_B+2, sizeof(double*))))
		errorExit("Cannot allocate space to LgammaB\n");
	for(i = 0; i <= MAX_B+1; i++) {
		if (!(LgammaB[i]=(double *) calloc(dimN+1, sizeof(double))))
			errorExit("Can not allocate space to LgammaB\n");
	}
	for(i = 0; i <= MAX_B+1; i++) {
        pseudocount = lambda*exp(-lambdaB*i);
        //pseudocount = lambda*exp(-log(nV)*i);
         
        LgammaB[i][0] = 0;
        for(unsigned int j = 1; j <= dimN; j++)
            LgammaB[i][j] = LgammaB[i][j-1] + log(j-1+pseudocount);
        } 

    if (!(LgammaY = (double *) calloc(MAX_GY+1, sizeof(double))))
		errorExit("Cannot allocate space to LgammaY\n");
	for(j = 0; j <= MAX_GY; j++)
		LgammaY[j] = alngam((dimN*j+nu)/2.0, &ifault) - alngam(nu/2.0, &ifault) - dimN*j*alngam(1.0/2.0, &ifault);
    
    if (!(LgammaC = (double *) calloc(dimN+1, sizeof(double))))
		errorExit("Cannot allocate space to LgammaY\n");
    LgammaC[0] = 0.0;
    for(j = 0; j < dimN; j++)
        LgammaC[0] += log(betaC+j);
    LgammaC[1] = 0.0;
    for(i = 2; i <= dimN; i++)
        LgammaC[i] = LgammaC[i-1]+log((double)(i-1));
    
	cout<<"Lgamma Initialized..."<<endl;
}

/**********************************
 *
 *calcSingleX
 *
 ************************************/
double calcSingleX( )
{
	unsigned int i,j,k;
	double logLik = 0.0, llk = 0.0;
	unsigned int * count = (unsigned int*) calloc(nV, sizeof(unsigned int));
	for(i = 0; i < dimP; i++)
	{
		for(k = 0; k < nV; k++)
			count[k]  = 0;
		for(j = 0; j < dimN; j++)
		{
			for(k = 0; k < nV; k++) 
			{
				if(X[dimN*i+j] == k)
					count[k] += 1;
			}
		}
		
		llk = - LgammaX[0][dimN];
		for(k = 0; k < nV; k++)
			llk +=  LgammaX[1][count[k]];
		logLikSingleX[i] = llk;
		logLik += llk;
	}	
	free(count);
	return logLik;
} 

/**********************************
 *
 *calcSingleY
 *
 ************************************/
double calcSingleY( )
{
	int ifault;
	unsigned int i,j,k;
	//double ssq = dimQ*dimN;
    double logLik = 0.0;
    float temp = 0.0;
	double constant = - (double)dimN/2.0 - (double)dimN*(0.5*log(2.0)+alngam(1.0/2.0, &ifault)); 
    double* mean = (double*) calloc(dimQ, sizeof(double));
    double* pt = Y;
    double* qt = NULL;

    for(i = 0; i < dimQ; i++)
    {
        double mean = 0;
        double sd = 0;
        qt = pt;
        for(j = 0; j < dimN; j++, pt++)
        {
            mean += *pt;
            sd += square(*pt);
        }
        mean = mean/(double)dimN;
        sd = sd/(double)dimN - mean*mean;
        sd = sqrt(sd);
        for(j = 0; j < dimN; j++, qt++)
        {
            (*qt) = ((*qt)-mean)/sd;
        }
        logLikSingleY[i] = constant;
        logLik += logLikSingleY[i];
    }
                                                               
    FILE *oFile;
    strcpy(fileName, outputDir);
	strcat(fileName,"/scale.score.txt");
	oFile = fopen (fileName, "w");
    for(i = 0; i < dimQ; i++)
    {	
        for(j = 0; j < dimN; j++)
        {
            fprintf(oFile,"%f\t",Y[i*dimN+j]);
        }
        fprintf(oFile,"\n");
    }
    fclose(oFile);
    free(mean);
    return logLik;
}

/**********************************
 *
 *calcBlock
 *
 ************************************/
void calcBlock( )
{
	//logLikBlockX[dimP][MAX_B+1]
    
	unsigned int i,j,k,v,w,x,c,m,n,start,limit,nCount;
	int jj;
	double llk = 0.0;
	int *map;
	if (!(map=(int *)calloc(nV, sizeof(int))))
		errorExit("In calcBlock, can't allocate space for map\n");
	unsigned int *Count;
	if (!(Count=(unsigned int *)calloc(dimN, sizeof(unsigned int))))
		errorExit("In calcBlock, can't allocate space for Count\n");
	unsigned int **Tree;
	if (!(Tree=(unsigned int **)calloc(dimN, sizeof(unsigned int*))))
		errorExit("In calcBlock, can't allocate space for Tree\n");
	for( i =0; i < dimN; i++)
	{
		if (!(Tree[i]=(unsigned int *)calloc(dimN, sizeof(unsigned int))))
			errorExit("In calcBlock, can't allocate space for Tree\n");
	}
    
	start = 0;
	for( i = 0; i < dimP; i++)
	{
		logLikBlockX[i][0] = logLikSingleX[i];
		nCount = 0;
		for(v = 0; v < nV; v++)
			map[v] = -1;
		for( k = 0; k < dimN; k++)
			Count[k] = 0;
		for( k = 0; k < dimN; k++)
		{
			x = X[dimN*i+k];
			if(map[x] == -1) 
			{
				map[x] = nCount;
				nCount += 1;
			}
			w = map[x];
			Tree[w][Count[w]] = k;
			Count[w] += 1;
		}
		
		if(blockBound[i] == 1)
			start = i;
		limit = start;
		if(i > start+MAX_B)
			limit = i-MAX_B;
        
		for( j = 1; j <= i-limit; j++)
		{
			jj = i-j;
            m = nCount;
			for(c = 0; c < m; c++) 
			{
				n = Count[c];
				if(n > 1)
				{
					for(v = 0; v < nV; v++)
						map[v] = -1;
					for( k = 0; k < n; k++)
					{
						x = X[dimN*jj+Tree[c][k]];
						if(k == 0)
						{
							map[x] = c;
							Count[c] = 0;
						}
						else if(map[x] == -1)
						{
							map[x] = nCount;
							nCount += 1;
						}
						w = map[x];
						Tree[w][Count[w]] = Tree[c][k];
						Count[w] += 1;
					}
				}
			}
            
            llk = -LgammaB[0][dimN];
			for(c = 0; c < nCount; c++) 
			{
				llk += LgammaB[j+1][Count[c]];
			}
			logLikBlockX[i][j] = llk;
		}
	}
    
	free(map);
	free(Count);
	for(i = 0; i < dimN; i++)
		free(Tree[i]);
	free(Tree);
}

/**********************************
 *
 *initBlock
 *
 ************************************/
void initBlock( double betaB )
{
	unsigned int i,j,n,start,limit;
	double llk = 0.0, max = 0.0;
    
	start = 0;
	for( i = 0; i < dimP; i++)
	{
		if(blockBound[i] == 1)
		{
			start = i;
			max = logLikBlockX[i][0];
		} else {
			max = logLikBlockX[i][0] + logLikEndX[i-1];	
		}
		limit = start;
		if(i > start+MAX_B)
			limit = i-MAX_B;
		n = i-limit;
		for( j = 1; j <= n; j++)
		{
            if( i-j > 0)
                llk = logLikBlockX[i][j] + logLikEndX[i-j-1];
            else
                llk = logLikBlockX[i][j];        
			if(llk > max)
				max = llk;
		}
		llk = 0.0;
		if(blockBound[i] == 1)
		{
			llk += exp(logLikBlockX[i][0]-max);
		} else {
			llk += exp(logLikBlockX[i][0]+logLikEndX[i-1]-max);	
		}
		for( j = 1; j <= n; j++)
		{
            if( i-j > 0)
                llk += exp(logLikBlockX[i][j]+logLikEndX[i-j-1]-max);
            else
                llk += exp(logLikBlockX[i][j]-max);
		}
		logLikEndX[i] = log(llk) + max + log(betaB);
		//cout << logLikEndX[i] << endl;
	}
}

/**********************************
 *
 *updateBlock
 *
 ************************************/
double updateBlock( int output )
{
	int i;
	unsigned int j,k,n;
    double logLikB = 0.0;
	double *logLik;
	if (!(logLik=(double *)calloc(MAX_B+1, sizeof(double))))
		errorExit("In updateBlock, can't allocate space for logLik	\n");
	
    for(i = 0; i < dimP; i++)
    {
        if(blockBound[i] == 2)
            blockBound[i] = 0;
    }
    
	i = dimP;
    logLikB = 0.0;
	while(i > 0)
	{
		i = i - 1;
		n = 0;
		for( j = 0; j <= MAX_B; j++)
		{
			k = i-j;
			n += 1;
			if(blockBound[k] == 1)
			{
				logLik[j] = logLikBlockX[i][j];
				break;
			} else {
				logLik[j] = logLikEndX[k-1] + logLikBlockX[i][j];
			}
		}
		k = multiSample(logLik,n);
        if(k >= n)
        {
            for(j = 0; j < n; j++)
                cout << logLik[j] << endl;
            errorExit("NaN!\n"); 
        }
        
		if(blockBound[i-k] == 0)
			blockBound[i-k] = 2;
        if(output == 1)
            resultB[i-k] += 1.0;
        logLikB += logLikBlockX[i][k];
		i = i - k;
		if(i < 0)
		{
			errorExit("Invalid index for positon!");
		}
	}
    return(logLikB);
}

/**********************************
 *
 *updateGroupY
 *
 ************************************/
void updateGroupY( struct GroupY* groupY, struct Record *record, double betaGY, int output )
{
	struct GroupY* temp;
    unsigned int i,j,k,g,c;
    unsigned int yG,nY;    
	double ssqY, ssqC, lik, likC, norm, kappa, zeta; 
	double *logLik, *sumC;
    unsigned int *indicator;
	if (!(logLik=(double *)calloc(nG+1, sizeof(double))))
		errorExit("In updateGroupY, can't allocate space for logLik\n");
	if (!(sumC=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateGroupY, can't allocate space for sumY\n");
    if (!(indicator=(unsigned int *)calloc(nG+1, sizeof(unsigned int))))
		errorExit("In updateGroupY, can't allocate space for indicator\n");

    double change = 0.0;
	for(i = 0; i < dimQ; i++)
	{
		logLik[0] = logLikSingleY[i];
        indicator[0] = 1;
		yG = record->yG[i];
        temp = groupY;
		for(g = 1; g < nG+1; g++, temp++)
		{
            kappa = temp->kappa;
            zeta = temp->zeta;
            indicator[g] = 1;
			if(yG == g) 
			{
                ssqY = 0.0, ssqC = 0.0, likC = 0.0;
				nY = temp->nY-1;
				for(j = 0; j < dimN; j++)
					ssqY += square(temp->sumY[j]-Y[dimN*i+j]) / (nY+kappa);

                for(j = 0; j < MAX_CL; j++)
                    sumC[j] = 0.0;
                for(c = 0; c < temp->nCL; c++)
                    sumC[temp->cID[c]] = temp->sumC[temp->cID[c]];
                for(j = 0; j < dimN; j++)
                    sumC[temp->sI[j]] -= Y[dimN*i+j];
                
                for(c = 0; c < temp->nCL; c++)
                {
                    norm = temp->cN[temp->cID[c]]*nY*kappa + (nY+kappa)*zeta;
                    likC += log(nY+kappa) + log(zeta)-log(norm);
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    ssqC += square(sumC[temp->cID[c]])*norm;
				}
                lik = 0.5*nu*(2.0*log(tau)+log(nu))+0.5*likC;
				lik += 0.5*dimN*(log(kappa)-log(kappa+nY))+LgammaY[nY];
				lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssqC);
				logLik[g] = temp->logLikelihood - lik - nY*log(betaGY);
			} else {
				ssqY = 0.0, ssqC = 0.0, likC = 0.0;
				nY = temp->nY+1;
                if(nY > MAX_GY)
                {
                    indicator[g] = 0;
                    logLik[g] = 0.0;
                } else
                {
                    for(j = 0; j < dimN; j++)
                        ssqY += square(temp->sumY[j]+Y[dimN*i+j]) / (nY+kappa);
                    for(j = 0; j < MAX_CL; j++)
                        sumC[j] = 0.0;
                    for(c = 0; c < temp->nCL; c++)
                        sumC[temp->cID[c]] = temp->sumC[temp->cID[c]];
                    for(j = 0; j < dimN; j++)
                        sumC[temp->sI[j]] += Y[dimN*i+j];
                    for(c = 0; c < temp->nCL; c++)
                    {
                        norm = temp->cN[temp->cID[c]]*nY*kappa + (nY+kappa)*zeta;
                        likC += log(nY+kappa) + log(zeta)-log(norm);
                        norm *= (nY + kappa);
                        norm = square(kappa)/norm;
                        ssqC += square(sumC[temp->cID[c]])*norm;
                    }
                    lik = 0.5*nu*(2.0*log(tau)+log(nu))+0.5*likC;
                    lik += 0.5*dimN*(log(kappa)-log(kappa+nY))+LgammaY[nY];
                    lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssqC);
                    logLik[g] = lik - temp->logLikelihood + nY*log(betaGY);                    
                }
			}
		}
        
		k = multiSample(logLik, nG+1, indicator);
        if(k >= nG+1)
        {
            for(j = 0; j < nG+1; j++)
                cout << logLik[j] << endl;
            errorExit("NaN!\n"); 
        }
        if(yG != k)
            change += logLik[k]-logLik[yG];
        
		if( yG != k ) {
			if( yG > 0 ) {
				temp = groupY+yG-1;
				nY = temp->nY-1;
                ssqY = 0.0, ssqC = 0.0;
                for(j = 0; j < dimN; j++)
                {
					temp->sumY[j] -= Y[dimN*i+j];
                    temp->sumC[temp->sI[j]] -= Y[dimN*i+j];
                    ssqY += square(temp->sumY[j]) / (nY+kappa);
                }
                for(c=0; c < temp->nCL; c++)
                {
                    norm = temp->cN[temp->cID[c]]*nY*kappa + (nY+kappa)*zeta;
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    ssqC += square(temp->sumC[temp->cID[c]]) * norm;
                }
                temp->nY -= 1;
                temp->ssqY = ssqY;
                temp->ssqC = ssqC;
				temp->logLikelihood -= logLik[yG];
			} else {
                record->logLikNullY -= logLik[0];
            }
            if( k > 0 ) {
				temp = groupY+k-1;
                nY = temp->nY+1;
                ssqY = 0.0, ssqC = 0.0;
                for(j = 0; j < dimN; j++)
                {
                    temp->sumY[j] += Y[dimN*i+j];
                    temp->sumC[temp->sI[j]] += Y[dimN*i+j];
                    ssqY += square(temp->sumY[j]) / (nY+kappa);
                }
                for(c=0; c < temp->nCL; c++)
                {
                    norm = temp->cN[temp->cID[c]]*nY*kappa + (nY+kappa)*zeta;
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    ssqC += square(temp->sumC[temp->cID[c]]) * norm;
                }    
                temp->nY += 1;
                temp->ssqY = ssqY;
                temp->ssqC = ssqC;
                temp->logLikelihood += logLik[k];
                record->yG[i] = k;
            } else {
                record->yG[i] = k;
                record->logLikNullY += logLik[0];
            }
        }
        if(record->yG[i] > 0 && output == 1)
        {
            resultGY[i][record->yG[i]] += 1.0;
            //cout <<  i << "\t" << k << "\t" << logLik[k] << "\t" << logLik[0] << endl;
        }
    }
    
    free(logLik);
    free(sumC);
    free(indicator);
}

/**********************************
 *
 *updateResponse
 *
 ************************************/
void updateResponse( struct GroupY *groupY, struct Module* module, struct Record *record, double betaY, double betaGY, int output )
{
    unsigned int i,j,k,d,g,c;
    unsigned int mID, gID, nY, reserve;    
	double ssqY, ssqC, lik, likC, norm, sum, kappa, zeta; 
    double *logLik, *sumC;
    unsigned int *indicator;
	if (!(logLik=(double *)calloc(nG+1, sizeof(double))))
		errorExit("In updateResponse, can't allocate space for logLik\n");
	if (!(sumC=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateResponse, can't allocate space for sumY\n");
    if (!(indicator=(unsigned int *)calloc(nG+1, sizeof(unsigned int))))
		errorExit("In updateResponse, can't allocate space for indicator\n");
    
    struct Module* temp = module;
    struct GroupY* tempY;
    for(d = 0; d < MAX_Y; d++)
    {
        indicator[0] = 1;
        logLik[0] = 0;
        tempY = groupY;
        gID = 0;
        for(g = 0; g < nG; g++, tempY++)
        {
            mID = tempY->moduleID;
            nY = tempY->nY;
            ssqY = tempY->ssqY;
            kappa = tempY->kappa;
            zeta = tempY->zeta;
            if(mID > 0 && mID != d+1)
            {
                indicator[g+1] = 0;
                logLik[g+1] = 0.0;
            } else if(mID == d+1) {
                gID = g+1;
                indicator[g+1] = 1;
                logLik[g+1] = tempY->logLikelihood+log(betaY);
                sum = 0.0;
                for(j = 0; j < dimN; j++)
                    sum += tempY->sumY[j];
                norm = dimN*nY*kappa + (nY+kappa)*zeta;
                likC = log(nY+kappa) + log(zeta)-log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssqC = square(sum)*norm;
                
                lik = 0.5*nu*(2.0*log(tau)+log(nu))+0.5*likC;
                lik += 0.5*dimN*(log(kappa)-log(kappa+nY))+LgammaY[nY];
                lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssqC);
                logLik[g+1] -= lik+nY*log(betaGY);
            } else {
                indicator[g+1] = 1;
                logLik[g+1] = -tempY->logLikelihood;
                
                for(j = 0; j < MAX_CL; j++)
                    sumC[j] = 0.0;
                for(j = 0; j < dimN; j++)
                    sumC[temp->sI[j]] += tempY->sumY[j];
                likC = 0.0, ssqC = 0.0;
                for(c = 0; c < temp->nCL; c++)
                {
                    norm = temp->cN[temp->cID[c]]*nY*kappa + (nY+kappa)*zeta;
                    likC += log(nY+kappa) + log(zeta)-log(norm);
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    ssqC += square(sumC[temp->cID[c]])*norm;
                }
                lik = 0.5*nu*(2.0*log(tau)+log(nu))+0.5*likC;
                lik += 0.5*dimN*(log(kappa)-log(kappa+nY))+LgammaY[nY];
                lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssqC);
                logLik[g+1] += lik+nY*log(betaGY)+log(betaY);         
            }
        }
        
        k = multiSample(logLik, nG+1, indicator);
        if(k >= nG+1)
        {
            for(j = 0; j < nG+1; j++)
                cout << logLik[j] << endl;
            errorExit("NaN!\n"); 
        }
        if( gID != k ) {
            reserve = 0;
            if(gID > 0)
            {
                if(temp->nYG == 1)
                    reserve = 1;
            }
            if( gID > 0 && reserve == 0 ) 
            {
                tempY = groupY+gID-1;
                nY = tempY->nY;
                ssqY = tempY->ssqY;
                kappa = tempY->kappa;
                zeta = tempY->zeta;
                
                for(i = 0; i < temp->nYG; i++)
                {
                    if(temp->groupYID[i] == gID)
                    {
                        temp->groupYID[i] = temp->groupYID[temp->nYG-1];
                        break;
                    }
                }
                if( i == temp->nYG )
                {
                    for(i = 0; i < temp->nYG; i++)
                    {
                        cout << temp->groupYID[i] << endl;
                    }
                    cout << k << "\t" << gID << endl;
                    errorExit("Group does not exist!\n");
                }
                temp->groupYID[temp->nYG-1] = 0;
                temp->nYG -= 1;
                temp->nY -= tempY->nY;
                temp->logLikY -= (tempY->logLikelihood+log(betaY));
                temp->logLik -= (tempY->logLikelihood+log(betaY)); 
                
                tempY->nCL = 1;
                for (c = 0; c < MAX_CL; c++)
                {
                    tempY->cID[c] = c;
                    tempY->cN[c] = 0;
                    tempY->sumC[c] = 0.0;
                }
                for (j = 0; j < dimN; j++){
                    tempY->sI[j] = 0;
                    tempY->cN[0] += 1;
                    tempY->sumC[0] += tempY->sumY[j];
                }
                norm = dimN*nY*kappa + (nY+kappa)*zeta;
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                tempY->ssqC = square(tempY->sumC[0])*norm;
                
                tempY->logLikelihood -= logLik[gID]-log(betaY);
                tempY->moduleID = 0;
                record->yI[gID-1] = 0;
            }
            
            if( k > 0 && reserve == 0 ) {
                tempY = groupY+k-1;
                nY = tempY->nY;
                ssqY = tempY->ssqY;
                kappa = tempY->kappa;
                zeta = tempY->zeta;
                
                temp->groupYID[temp->nYG] = k;
                temp->nYG += 1;
                temp->nY += tempY->nY;
                temp->logLikY += logLik[k]+tempY->logLikelihood;
                temp->logLik += logLik[k]+tempY->logLikelihood;
                
                tempY->nCL = temp->nCL;
                for(c = 0; c < MAX_CL; c++)
                {
                    tempY->cID[c] = temp->cID[c];
                    tempY->cN[c] = temp->cN[c];
                    tempY->sumC[c] = 0.0;
                }
                for(j = 0; j < dimN; j++)
                {
                    tempY->sI[j] = temp->sI[j];
                    tempY->sumC[temp->sI[j]] += tempY->sumY[j];
                }
                tempY->ssqC = 0.0;
                for(c = 0; c < tempY->nCL; c++)
                {
                    norm = tempY->cN[tempY->cID[c]]*nY*kappa + (nY+kappa)*zeta;
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    tempY->ssqC += square(tempY->sumC[tempY->cID[c]])*norm;
                }
                tempY->logLikelihood += logLik[k]-log(betaY);
                tempY->moduleID = d+1;
                record->yI[k-1] = 1;
            }
        }
        // cout << "ResponseGroup " << g << " Module " << record->yI[g] << endl;
    }
    
    for(g = 0; g < nG; g++)
    {
        if(record->yI[g]>0 && output == 1) 
        {
            resultY[g][record->yI[g]] += 1.0;
            //	cout <<  g << "\t" << k << "\t" << logLik[k] << "\t" << logLik[0] << endl;
        } 
    }
    free(logLik);
    free(sumC);
    free(indicator);
}


/**********************************
*
*updateClusterY
*
************************************/
void updateClusterY( struct GroupY* groupY, double betaC )
{
    unsigned int i,j,k,g,c,n;
    unsigned int sI, cN, nCL, nY=groupY->nY, kID, cID, addNew;
    double norm, ssq, ssqC, logLikC, likC, sumY, ssqY=groupY->ssqY, kappa=groupY->kappa, zeta=groupY->zeta;
    double *logLik;
	if (!(logLik=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for logLik\n");
    double *logLikD;
	if (!(logLikD=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for logLik\n");

	for( i = 0; i < dimN; i++)
	{
		sI = groupY->sI[i];
		cN = groupY->cN[sI];
		nCL = groupY->nCL;
        sumY = groupY->sumY[i];
        
        norm = cN*nY*kappa + (nY+kappa)*zeta;
        logLikC = 0.5*log(norm);
        norm *= (nY + kappa);
        norm = square(kappa)/norm;
        ssqC = groupY->ssqC-square(groupY->sumC[sI]) * norm;
        norm = (cN-1)*nY*kappa + (nY+kappa)*zeta;
        logLikC -= 0.5*log(norm);
        norm *= (nY + kappa);
        norm = square(kappa)/norm;
        ssqC += square(groupY->sumC[sI]-sumY) * norm;
        if(cN > 1)
            likC = -log((double)(cN-1));
        else 
            likC = -log(betaC);
        logLikC += likC;
        logLikC += 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-groupY->ssqC);

		for(c = 0; c < nCL; c++)
		{
            cID = groupY->cID[c];
			if(cID == sI)
            {
                logLik[c] = 0.0;
                logLikD[c] = 0.0;
            } else {
				n = groupY->cN[cID];
                ssq = ssqC;
                if(n > 0)
                {
                    logLikD[c] = likC+log((double)(n));
                    logLik[c] = logLikC+log((double)(n));
                }
                norm = n*nY*kappa + (nY+kappa)*zeta;
                logLik[c] += 0.5*log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssq -= square(groupY->sumC[cID]) * norm;
                norm = (n+1)*nY*kappa + (nY+kappa)*zeta;
                logLik[c] -= 0.5*log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssq += square(groupY->sumC[cID]+sumY) * norm;
                logLik[c] -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssq);
            }
		}
        addNew = 0;
		if( nCL < MAX_CL && cN > 1 ) {
            ssq = ssqC;
            logLikD[nCL] = likC+log(betaC);
            logLik[nCL] = logLikC+log(betaC);
            norm = (nY+kappa)*zeta;
            logLik[nCL] += 0.5*log(norm);
            norm = 1*nY*kappa + (nY+kappa)*zeta;
            logLik[nCL] -= 0.5*log(norm);
            norm *= (nY + kappa);
            norm = square(kappa)/norm;
            ssq += square(sumY) * norm;
            logLik[nCL] -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssq);
            nCL += 1;
            addNew = 1;
		}
		k = multiSample(logLik, nCL);
        if(k >= nCL)
        {
            for(j = 0; j < nCL; j++)
                cout << logLik[j] << endl;
            errorExit("NaN!\n"); 
        }
		kID = groupY->cID[k];
		if(addNew == 1 && k == nCL-1) 
		{ //create new cluster
            groupY->sumC[sI] -= sumY;
            groupY->cN[sI] -= 1;
            
            groupY->sumC[kID] = sumY;
            groupY->cN[kID] = 1;
            groupY->sI[i] = kID;
            groupY->nCL += 1;
            
            norm = 1*nY*kappa + (nY+kappa)*zeta;
            norm *= (nY + kappa);
            norm = square(kappa)/norm;
            groupY->ssqC = ssqC + square(groupY->sumC[kID]) * norm;
            groupY->logLikelihood += logLik[k]-logLikD[k];
            groupY->logDirPrior += logLikD[k];
        } else if( kID != sI && !((groupY->nCL == 2) && (cN == 1)) ) { 
            // move to an existing cluster
            // do not move if there will be only one cluster
            
            groupY->sumC[sI] -= sumY;
            groupY->cN[sI] -= 1;
            
            norm = groupY->cN[kID]*nY*kappa + (nY+kappa)*zeta;
            norm *= (nY + kappa);
            norm = square(kappa)/norm;
            groupY->ssqC = ssqC - square(groupY->sumC[kID]) * norm;
            
            groupY->sumC[kID] += sumY;
            groupY->cN[kID] += 1;
            groupY->sI[i] = kID;
            
            norm = groupY->cN[kID]*nY*kappa + (nY+kappa)*zeta;
            norm *= (nY + kappa);
            norm = square(kappa)/norm;
            groupY->ssqC += square(groupY->sumC[kID]) * norm;
            groupY->logLikelihood += logLik[k]-logLikD[k];
            groupY->logDirPrior += logLikD[k];
			if(cN == 1) 
			{
				for(c = 0; c < groupY->nCL; c++) 
				{
					if(groupY->cID[c] == sI) 
					{
						groupY->cID[c] = groupY->cID[groupY->nCL-1];
						break;
					}
				}
				groupY->cID[groupY->nCL-1] = sI;
				groupY->nCL -= 1;
			}
		}
	}
    
    //cout<<"NumberOfResponseGroups: "<<groupY->nCL<<endl;
	//for(c = 0; c < groupY->nCL; c++)
    //	cout<<groupY->cN[groupY->cID[c]]<<endl;
	
	free(logLik);
    free(logLikD);
}


/**********************************
 *
 *updateCluster
 *
 ************************************/
void updateCluster( struct Module* module, struct GroupY* response, double betaC )
{
    unsigned int i,j,k,g,c,n;
    unsigned int sI, cN, nCL, nY, kID, cID, addNew, temp, code, cK, nYG=module->nYG, nXG=module->nXG;
    double norm, sumY, ssq, ssqY, lik, likY, likX, likC, kappa, zeta, pseudocount;
    
    struct GroupY* groupY;
    struct GroupX* groupX;
    double *logLik, *logLikD, *logLikX, *logLikY, *logLikYG, *ssqC;
	if (!(logLik=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for logLik\n");
    if (!(logLikX=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for logLikX\n");
    if (!(logLikY=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for logLikY\n");
    if (!(logLikD=(double *)calloc(MAX_CL, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for logLikD\n");
	if (!(ssqC=(double *)calloc(nYG, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for sumY\n");
    if (!(logLikYG=(double *)calloc(nYG, sizeof(double))))
		errorExit("In updateClusterY, can't allocate space for sumY\n");
        
	for( i = 0; i < dimN; i++)
	{
        sI = module->sI[i];
		cN = module->cN[sI];
		nCL = module->nCL;
        
        if(cN > 1)
            likC = -log((double)(cN-1));
        else
            likC = -log(betaC);
        likX = 0.0;
        likY = 0.0;
        for(g = 0; g < nYG; g++)
        {
            groupY = response + module->groupYID[g]-1;
            sumY = groupY->sumY[i], nY = groupY->nY, ssqY = groupY->ssqY;
            kappa = groupY->kappa, zeta = groupY->zeta;
            lik = 0.0;
            norm = cN*nY*kappa + (nY+kappa)*zeta;
            lik += 0.5*log(norm);
            norm *= (nY + kappa);
            norm = square(kappa)/norm;
            ssqC[g] = groupY->ssqC-square(groupY->sumC[sI]) * norm;
            norm = (cN-1)*nY*kappa + (nY+kappa)*zeta;
            lik -= 0.5*log(norm);
            norm *= (nY + kappa);
            norm = square(kappa)/norm;
            ssqC[g] += square(groupY->sumC[sI]-sumY) * norm;
            lik += 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-groupY->ssqC);  
            logLikYG[g] = lik;
            likY += lik;
        }
		for( g = 0; g < nXG; g++)
		{	
			groupX = module->groupX + module->groupXID[g];
			code = groupX->code[i];
			cK = groupX->cK[sI];	
			temp = 0;
			for(j = 0; j < cK; j++) 
			{
				if(groupX->pattern[sI][j] == code)
				{
					temp = groupX->count[sI][j];
					break;
				}
			}
			if( temp == 0)
				errorExit("Pattern does not exist!\n");
			pseudocount = lambda/(double)PowerX[groupX->nX];
			likX += log( cN - 1 + lambda) - log( temp - 1 + pseudocount);
		}     

		for(c = 0; c < nCL; c++)
		{
            cID = module->cID[c];
			if(cID == sI)
            {
                logLik[c] = 0.0;
                logLikX[c] = 0.0;
                logLikY[c] = 0.0;
            } else {
                n = module->cN[cID];
                if(n > 0)
                    logLikD[c] = likC+log((double)(n));
                logLik[c] = logLikD[c];
                lik = likY;
                for(g = 0; g < nYG; g++)
                {
                    groupY = response + module->groupYID[g]-1;
                    sumY = groupY->sumY[i], nY = groupY->nY, ssqY = groupY->ssqY;
                    kappa = groupY->kappa, zeta = groupY->zeta;
                    ssq = ssqC[g];
                    norm = n*nY*kappa + (nY+kappa)*zeta;
                    lik += 0.5*log(norm);
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    ssq -= square(groupY->sumC[cID]) * norm;
                    norm = (n+1)*nY*kappa + (nY+kappa)*zeta;
                    lik -= 0.5*log(norm);
                    norm *= (nY + kappa);
                    norm = square(kappa)/norm;
                    ssq += square(groupY->sumC[cID]+sumY) * norm;
                    lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssq);
                }
                logLikY[c] = lik;
                logLik[c] += lik;
                
                lik = likX;
                for( g = 0; g < nXG; g++)
				{
                    groupX = module->groupX + module->groupXID[g];
                    code = groupX->code[i];
                    cK = groupX->cK[cID];	
                    temp = 0;
                    for(j = 0; j < cK; j++) 
                    {
                        if(groupX->pattern[cID][j] == code)
                        {
                            temp = groupX->count[cID][j];
                            break;
                        }
                    }
                    pseudocount = lambda/(double)PowerX[groupX->nX];
                    lik += - log(n + lambda) + log(temp + pseudocount);
				}
                logLikX[c] = lik;
                logLik[c] += lik;
            }
		}
        addNew = 0;
		if( nCL < MAX_CL && cN > 1 ) {
            logLikD[nCL] = likC+log(betaC);
            logLik[nCL] = logLikD[nCL];
            lik = likY;
            for(g = 0; g < nYG; g++)
            {
                groupY = response + module->groupYID[g]-1;
                sumY = groupY->sumY[i], nY = groupY->nY, ssqY = groupY->ssqY;
                kappa = groupY->kappa, zeta = groupY->zeta;
                ssq = ssqC[g];
                norm = (nY+kappa)*zeta;
                lik += 0.5*log(norm);
                norm = 1*nY*kappa + (nY+kappa)*zeta;
                lik -= 0.5*log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssq += square(sumY) * norm;
                lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssq);
            }   
            logLikY[nCL] = lik;
            logLik[nCL] += lik;

            lik = likX;
            for( g = 0; g < nXG; g++)
			{
				groupX = module->groupX + module->groupXID[g];
                pseudocount = lambda/(double)PowerX[groupX->nX];
                lik += -log(lambda) + log(pseudocount);
            }
            logLikX[nCL] = lik;
            logLik[nCL] += lik;
            nCL += 1;
            addNew = 1;
		}
        
		k = multiSample(logLik, nCL);
        if(k >= nCL)
        {
            for(j = 0; j < nCL; j++)
                cout << logLik[j] << endl;
            errorExit("NaN!\n"); 
        }
		kID = module->cID[k];
		if(addNew == 1 && k == nCL-1) 
		{ //create new cluster
            for(g = 0; g < nYG; g++)
            {
                groupY = response + module->groupYID[g]-1;
                sumY = groupY->sumY[i], nY = groupY->nY, ssqY = groupY->ssqY;
                kappa = groupY->kappa, zeta = groupY->zeta;
                lik = logLikYG[g];
                ssq = ssqC[g];
                
                norm = (nY+kappa)*zeta;
                lik += 0.5*log(norm);

                groupY->sumC[sI] -= sumY;
                groupY->cN[sI] -= 1;
                groupY->sumC[kID] = sumY;
                groupY->cN[kID] = 1;
                groupY->sI[i] = kID;
                groupY->nCL += 1;

                norm = 1*nY*kappa + (nY+kappa)*zeta;
                lik -= 0.5*log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssq += square(sumY) * norm;
                lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssq);
                groupY->ssqC = ssq;
                groupY->logLikelihood += lik;
            }
            
            for( g = 0; g < nXG; g++)
			{
				groupX = module->groupX + module->groupXID[g];                
                code = groupX->code[i];
				cK = groupX->cK[sI];
                temp = 0;
                lik = 0.0;
                pseudocount = lambda/(double)PowerX[groupX->nX];
				for(j = 0; j < cK; j++) 
				{
					if(groupX->pattern[sI][j] == code) 
					{
                        temp = groupX->count[sI][j];
						groupX->count[sI][j] -= 1;
						break;
					}
				}
				if( j == cK)
					errorExit("Pattern does not exist!\n");
                lik += log( cN - 1 + lambda) - log( temp - 1 + pseudocount);
                
				if(groupX->count[sI][j] == 0) 
				{
					groupX->pattern[sI][j] = groupX->pattern[sI][cK-1];
					groupX->count[sI][j] = groupX->count[sI][cK-1];
					groupX->pattern[sI][cK-1] = 0;
					groupX->count[sI][cK-1] = 0;
					groupX->cK[sI]--;
				}
                
                lik += - log(lambda) + log(pseudocount);
				groupX->pattern[kID][0] = code;
				groupX->count[kID][0] = 1;
				groupX->cK[kID] = 1;
                groupX->logLikelihood += lik;
			}
            
            module->cN[sI] -= 1;
            module->cN[kID] = 1;
            module->sI[i] = kID;
            module->nCL += 1;
            
            module->logLik += logLik[k];
            module->logLikX += logLikX[k];
            module->logLikY += logLikY[k];
            module->logDirPrior += logLikD[k];
        } else if( kID != sI && !((groupY->nCL == 2) && (cN == 1)) ) { 
            // move to an existing cluster
            // do not move if there will be only one cluster
            
            for(g = 0; g < nYG; g++)
            {
                groupY = response + module->groupYID[g]-1;
                sumY = groupY->sumY[i], nY = groupY->nY, ssqY = groupY->ssqY;
                kappa = groupY->kappa, zeta = groupY->zeta;
                lik = logLikYG[g];
                ssq = ssqC[g];
                
                norm = groupY->cN[kID]*nY*kappa + (nY+kappa)*zeta;
                lik += 0.5*log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssq -= square(groupY->sumC[kID]) * norm;
            
                groupY->sumC[sI] -= sumY;
                groupY->cN[sI] -= 1;
                groupY->sumC[kID] += sumY;
                groupY->cN[kID] += 1;
                groupY->sI[i] = kID;
            
                norm = groupY->cN[kID]*nY*kappa + (nY+kappa)*zeta;
                lik -= 0.5*log(norm);
                norm *= (nY + kappa);
                norm = square(kappa)/norm;
                ssq += square(groupY->sumC[kID]) * norm;
                lik -= 0.5*(dimN*nY+nu)*log(nu*square(tau)+dimN*nY-ssqY-ssq);
                groupY->ssqC = ssq;
                groupY->logLikelihood += lik;
                if(cN == 1) 
                {       
                    for(c = 0; c < groupY->nCL; c++) 
                    {
                        if(groupY->cID[c] == sI) 
                        {
                            groupY->cID[c] = groupY->cID[groupY->nCL-1];
                            break;
                        }
                    }
                    groupY->cID[groupY->nCL-1] = sI;
                    groupY->nCL -= 1;
                }
            }
		
            n = module->cN[kID];
            for( g = 0; g < nXG; g++)
            {
                groupX = module->groupX + module->groupXID[g];
                code = groupX->code[i];
                cK = groupX->cK[sI];
                temp = 0;
                lik = 0.0;
                pseudocount = lambda/(double)PowerX[groupX->nX];
                for(j = 0; j < cK; j++) 
                {
                    if(groupX->pattern[sI][j] == code) 
                    {
                        temp = groupX->count[sI][j];
                        groupX->count[sI][j] -= 1;
                        break;
                    }
                }
                if( j == cK)
                    errorExit("Pattern does not exist!\n");
                if(groupX->count[sI][j] == 0) 
                {
                    groupX->pattern[sI][j] = groupX->pattern[sI][cK-1];
                    groupX->count[sI][j] = groupX->count[sI][cK-1];
                    groupX->pattern[sI][cK-1] = 0;
                    groupX->count[sI][cK-1] = 0;
                    groupX->cK[sI]--;
                }
                lik += log( cN - 1 + lambda) - log( temp - 1 + pseudocount);
 
                cK = groupX->cK[kID];
                temp = 0;
                for(j = 0; j < cK; j++) 
                {
                    if(groupX->pattern[kID][j] == code)
                    {
                        temp = groupX->count[kID][j];
                        groupX->count[kID][j] += 1;
                        break;
                    }
                }
                if(j == cK) 
				{
					groupX->pattern[kID][j] = code;
					groupX->count[kID][j] = 1;
					groupX->cK[kID]++;
				}
                lik += - log(n + lambda) + log(temp + pseudocount);
                groupX->logLikelihood += lik;
            }
        
            module->cN[sI] -= 1;
            module->cN[kID] += 1;
            module->sI[i] = kID;       
            if(cN == 1) 
            {       
                for(c = 0; c < module->nCL; c++) 
                {
                    if(module->cID[c] == sI) 
                    {
                        module->cID[c] = module->cID[module->nCL-1];
                        break;
                    }
                }
                module->cID[module->nCL-1] = sI;
                module->nCL -= 1;
            }
        
            module->logLik += logLik[k];
            module->logLikX += logLikX[k];
            module->logLikY += logLikY[k];
            module->logDirPrior += logLikD[k];
        }
    }
    
    //cout<<"NumberOfGroups: "<<module->nCL<<endl;
	//for(c = 0; c < module->nCL; c++)
    //	cout<<module->cN[module->cID[c]]<<endl;
	
	free(logLik);
    free(logLikX);
    free(logLikY);
    free(logLikD);
    free(logLikYG);
	free(ssqC);
}

/**********************************
 *
 *initializeModules
 *
 ************************************/
void initializeModules( struct GroupY* groupY, struct Module *module, struct Record* record )
{
    struct GroupY *tempY = groupY;
    struct Record *tempR = record;
    struct Module *tempM = module;
    //struct GroupX *tempX;
    unsigned int i, t, g, c, d;
    
    /* dynamic memory allocation*/
	for( t = 0; t < nT; t++, tempR++)
	{
		for(d = 0; d < nD; d++, tempM++, tempY++)
        {
            tempM->chisi = tempR->chisi;
            tempM->logLik = 0.0;
            tempM->logLikX = 0.0;
            tempM->nYG = 1;
            tempM->nXG = 0;
            tempM->nX = 0;
            tempM->nY = tempY->nY;
            tempM->nCL = tempY->nCL;
            tempM->logDirPrior = tempY->logDirPrior;
            tempM->logLikY = tempY->logLikelihood+log(betaY);
            tempM->logLik = tempM->logLikY + tempM->logLikX + tempM->logDirPrior;
            tempY->logDirPrior = 0.0;

            if (!(tempM->sI = (unsigned int *) calloc(dimN, sizeof(unsigned int))))
				errorExit("cannot allocate space to sI\n");
            if (!(tempM->cID = (unsigned int *) calloc(MAX_CL, sizeof(unsigned int))))
				errorExit("cannot allocate space to cID\n");
            if (!(tempM->cN = (unsigned int *) calloc(MAX_CL, sizeof(unsigned int))))
				errorExit("cannot allocate space to cN\n");
            for(i = 0; i < dimN; i++)
                tempM->sI[i] = tempY->sI[i];
            for(c = 0; c < MAX_CL; c++)
            {
                tempM->cID[c] = tempY->cID[c];  
                tempM->cN[c] = tempY->cN[c];
            }
            if (!(tempM->groupYID = (unsigned int *) calloc(MAX_Y, sizeof(unsigned int))))
				errorExit("Can not allocate space for groupYID\n");
            for( g = 0; g < MAX_Y; g++)
                tempM->groupYID[g] = 0;            
            tempM->groupYID[0] = d+1;
            tempR->yI[d] = d+1;
             
        }
    }
    
}

/**********************************
 *
 *cleanUpModules
 *
 ************************************/
void cleanUpModules( struct Module *module )
{
    unsigned int i, t, d, g, c;
	struct Module* tempM = module;
	struct GroupX* tempX;
	for( t = 0; t < nT; t++ )
	{
		for( d = 0; d < nD; d++, tempM++)
		{
			free(tempM->sI);
			free(tempM->cID);
			free(tempM->cN);
			free(tempM->groupXID);
            free(tempM->groupYID);
        }
	}
}

/**********************************
 *
 *initializeGroupY
 *
 ************************************/
void initializeGroupY( GroupY *groupY, Record *record, double betaC )
{
    struct GroupY *tempY = groupY;
    struct Record *tempR = record;
    unsigned int i, t, g, c, k;   
    
    /* dynamic memory allocation*/
	for( t = 0; t < nT; t++, tempR++)
	{
		tempR->chisi = 1.0;
		tempR->logLikNullX = logLikNullX;
		tempR->logLikNullY = logLikNullY;
		tempR->logLik = tempR->logLikNullX+tempR->logLikNullY;
        tempR->logLikB = 0.0;
		tempR->xI = (unsigned int *) calloc(dimP, sizeof(unsigned int));
		tempR->xG = (unsigned int *) calloc(dimP, sizeof(unsigned int));	
        tempR->yI = (unsigned int *) calloc(nG, sizeof(unsigned int));
        tempR->yG = (unsigned int *) calloc(dimQ, sizeof(unsigned int));
        for(i = 0; i < dimP; i++)
		{
			tempR->xI[i] = 0;
			tempR->xG[i] = 0;
		}
		for(i = 0; i < dimQ; i++)
            tempR->yG[i] = 0;
        for(i = 0; i < nG; i++)
			tempR->yI[i] = 0;
        
		for( g = 0; g < nG; g++, tempY++)
		{
            tempY->kappa=kappa;
            tempY->zeta=zeta;
            tempY->nCL=0;
            tempY->nY=0;
            tempY->ssqY=0.0;
            tempY->ssqC=0.0;
            tempY->logLikelihood=0.0;
            tempY->logDirPrior=0.0;
            
            if (!(tempY->sumY = (double *) calloc(dimN, sizeof(double))))
				errorExit("cannot allocate space to sumY\n");
            if (!(tempY->sI = (unsigned int *) calloc(dimN, sizeof(unsigned int))))
				errorExit("cannot allocate space to sI\n");
            if (!(tempY->sumC = (double *) calloc(MAX_CL, sizeof(double))))
				errorExit("cannot allocate space to sumC\n");
            if (!(tempY->cID = (unsigned int *) calloc(MAX_CL, sizeof(unsigned int))))
				errorExit("cannot allocate space to cID\n");
            if (!(tempY->cN = (unsigned int *) calloc(MAX_CL, sizeof(unsigned int))))
				errorExit("cannot allocate space to cN\n");
        }
    }

    /*Initialize clusters for each ladder*/
	printf("Initialize.....\n");
	tempY = groupY;
    tempR = record;
	for (t = 0; t < nT; t++, tempR++)
	{
        for (g = 0; g < nD; g++, tempY++)
		{
			tempY->nCL = INIT_CL;
			for (c = 0; c < MAX_CL; c++)
			{
				tempY->cID[c] = c;
				tempY->cN[c] = 0;
                tempY->sumC[c] = 0.0;
			}
            tempY->ssqC = 0.0;
			for (i = 0; i < dimN; i++)
                tempY->sumY[i] = 0.0;
            tempY->ssqY = 0.0;
			for (i = 0; i < dimN; i++){
				c = (int) floor(my_uniform(1.0)*tempY->nCL);
				tempY->sI[i] = c;
				tempY->cN[c] += 1;
			}
			for(c = 0; c < tempY->nCL; c++)
			{
                if(tempY->cN[tempY->cID[c]] == 0)
                {
                    k = (int) floor(my_uniform(1.0)*dimN);
                    while ( tempY->cN[tempY->sI[k]] <= 1 )
                        k = (int) floor(my_uniform(1.0)*dimN);
                    tempY->cN[tempY->sI[k]] -= 1;
                    tempY->sI[k] = tempY->cID[c];
                    tempY->cN[tempY->cID[c]] += 1;
                }
			}
            tempY->logDirPrior = (tempY->nCL)*log(betaC) - LgammaC[0];
			for(c = 0; c < tempY->nCL; c++) {
                tempY->logDirPrior += LgammaC[tempY->cN[tempY->cID[c]]];
            }
            tempY->logLikelihood = 0.0;
            tempY->moduleID = g+1;
		}
        for (g = nD; g < nG; g++, tempY++)
		{
            tempY->nCL = 1;
            for (c = 0; c < MAX_CL; c++)
			{
				tempY->cID[c] = c;
				tempY->cN[c] = 0;
                tempY->sumC[c] = 0.0;
			}
            tempY->ssqC = 0.0;
			for (i = 0; i < dimN; i++)
                tempY->sumY[i] = 0.0;
            tempY->ssqY = 0.0;
			for (i = 0; i < dimN; i++){
				tempY->sI[i] = 0;
				tempY->cN[0] += 1;
			}
            tempY->logDirPrior = 0.0;
            tempY->logLikelihood = 0.0;
            tempY->moduleID = 0;
        }
	}
}


/**********************************
 *
 *cleanUpGroupY
 *
 ************************************/
void cleanUpGroupY( GroupY *groupY, Record *record )
{
    struct GroupY *tempY=groupY;
    struct Record *tempR=record;
    unsigned int t, g;
    
    /* dynamic memory allocation*/
	for( t = 0; t < nT; t++, tempR++)
	{
		free(tempR->xI);
        free(tempR->xG);
        free(tempR->yI);
        free(tempR->yG);
        
		for( g = 0; g < nG; g++, tempY++)
		{
            free(tempY->sumY);
            free(tempY->sI);
            free(tempY->sumC);
            free(tempY->cID);
            free(tempY->cN);
        }
    }
}
    
/**********************************
 *
 *readResponse
 *
 ************************************/
void readResponse( )
{
	unsigned int i,j;
	float floattemp = 0.0, sum = 0.0, ssq = 0.0;
	int temp = 0;
    
	FILE * iFile;
	strcpy(fileName, inputDir);
	iFile = fopen(fileName, "r");
    for(i = 0; i < dimQ; i++)
	{	
		for(j = 0; j < dimN; j++)
		{
			fscanf (iFile, "%f", &floattemp);
			Y[i*dimN+j] = floattemp;
            sum += floattemp;
            ssq += square(floattemp);
		}
	}
    meanAll = sum/(double)(dimQ*dimN);
	ssq = ssq/(double)(dimQ*dimN) - square(meanAll);
	sdAll = sqrt(ssq);
	fclose (iFile);    
}

/**********************************
*
*writeResultModules
*
************************************/
void writeResultModules( double **resultX, double **resultY, double **resultGX, double **resultGY, int **resultC, double *resultB)
{
    int i,j,n = nBurn;
    double sum = 0.0;
    FILE *yFile;
    strcpy(fileName, outputDir);
	strcat(fileName,"/resulty.txt");
	yFile = fopen (fileName, "w");
    for(i = 0; i < nG; i++)
	{	
        sum = 0.0;
        for(j = 1; j < nD+1; j++)
		{
            sum += resultY[i][j];
		}
        resultY[i][0] = n-sum;
		for(j = 0; j < nD+1; j++)
		{
			fprintf(yFile,"%f\t",resultY[i][j]/(double)n);
		}
		fprintf(yFile,"\n");
	}
    fclose(yFile);

    FILE *ygFile;
    strcpy(fileName, outputDir);
	strcat(fileName,"/resultyg.txt");
	ygFile = fopen (fileName, "w");
    for(i = 0; i < dimQ; i++)
	{	
        sum = 0.0;
        for(j = 1; j < nG+1; j++)
		{
            sum += resultGY[i][j];
		}
        resultGY[i][0] = (double)n/(double)nGap-sum;
		for(j = 0; j < nG+1; j++)
		{
			fprintf(ygFile,"%f\t",(double)nGap*resultGY[i][j]/(double)n);
		}
		fprintf(ygFile,"\n");
	}
    fclose(ygFile);
    
    FILE *cFile;
    strcpy(fileName, outputDir);
	strcat(fileName,"/resultc.txt");
	cFile = fopen (fileName, "w");
    for(i = 0; i < nBurn; i++)
	{	
		for(j = 0; j < dimN; j++)
		{
			fprintf(cFile,"%d\t",resultC[i][j]);
		}
		fprintf(cFile,"\n");
	}
    fclose(cFile);
}
    
/**********************************
 *
 *writeResultGroupY
 *
 ************************************/
void writeResultGroupY( double **resultGY, int **resultCY )
{
    int i,j;
    double sum = 0.0;
    FILE *gyFile;
    strcpy(fileName, outputDir);
	strcat(fileName,"/resultgy.txt");
	gyFile = fopen (fileName, "w");
    for(i = 0; i < dimQ; i++)
	{	
        sum = 0.0;
        for(j = 1; j < nG+1; j++)
		{
            sum += resultGY[i][j];
		}
        resultGY[i][0] = nInit-sum;
		for(j = 0; j < nG+1; j++)
		{
			fprintf(gyFile,"%f\t",resultGY[i][j]/(double)nInit);
		}
		fprintf(gyFile,"\n");
	}
    fclose(gyFile);
    
    FILE *cyFile;
    strcpy(fileName, outputDir);
	strcat(fileName,"/resultcy.txt");
	cyFile = fopen (fileName, "w");
    for(i = 0; i < nInit; i++)
	{	
		for(j = 0; j < dimN; j++)
		{
			fprintf(cyFile,"%d\t",resultCY[i][j]);
		}
		fprintf(cyFile,"\n");
	}
    fclose(cyFile);
}
