#########################
##Find Preferred 6-mers##
#########################
preferred.pvalue <- function(dir, file1, file2, mu1=2.0, mu2=0.0, level=0.05, p.adjust.method="fdr",head=TRUE)
{
    setwd(dir)
    
    kmer.6.f <- kmer.6$V1
    kmer.6.r <- kmer.6$V2
    n <- length(kmer.6.f)
    
    data1 <- read.table(file1,header=F,sep="\t")
    x1 <- as.vector(data1$V3)
    x1 <- as.numeric(x1[2:length(x1)])
    kmers.f <- as.vector(data1$V1)
    kmers.f <- kmers.f[2:length(kmers.f)]
    kmers.r <- as.vector(data1$V2)
    kmers.r <- kmers.r[2:length(kmers.r)]
    kmers <- cbind(kmers.f,kmers.r)
    data2 <- read.table(file2,header=F,sep="\t")
    x2 <- as.vector(data2$V3)
    x2 <- as.numeric(x2[2:length(x2)])
    #x1 <- x1-tau*s
    #x2 <- x2-tau*s
    
    x12 <- x1-x2
    x1 <- (x1-mean(x1))/sd(x1)
    x2 <- (x2-mean(x2))/sd(x2)
    x12 <- (x12-mean(x12))/sd(x12)
    x21 <- -x12 
    
    pval.1 <- rep(1,n)
    pval.2 <- rep(1,n)
    mean.1 <- rep(0,n)
    diff.1 <- rep(0,n)
    mean.2 <- rep(0,n)
    diff.2 <- rep(0,n)
    m <- rep(0,n)
    for(i in 1:n)
    {
        id1 <- grep(kmer.6.f[i],as.character(kmers[,1]))
        id2 <- grep(kmer.6.f[i],as.character(kmers[,2]))
        id <- unique(c(id1,id2))
        m[i] <- length(id1)
        t1 <- t.test(x1[id],mu=mu1,alternative="greater")
        t2 <- t.test(x2[id],mu=mu1,alternative="greater")
        t12 <- t.test(x12[id],mu=mu2,alternative="greater")
        t21 <- t.test(x21[id],mu=mu2,alternative="greater")
        #t12 <- t.test(x12[id],x12[-id],alternative="greater")
        #t21 <- t.test(x21[id],x21[-id],alternative="greater")
        pval.1[i] <- max(t1$p.value,t12$p.value)
        pval.2[i] <- max(t2$p.value,t21$p.value)
        mean.1[i] <- mean(x1[id])
        diff.1[i] <- mean(x12[id]) 
        #diff.1[i] <- -diff(t12$estimate)
        mean.2[i] <- mean(x2[id])
        diff.2[i] <- mean(x21[id])
        #diff.2[i] <- -diff(t21$estimate)
    }
    
    padj.1 <- p.adjust(pval.1, method=p.adjust.method)
    k1 <- sum(padj.1<level)
    preferred.1 <- NULL
    if(k1 > 0)
    {
        id1 <- sort(pval.1,decreasing=F,index.return=T)$ix[1:k1]
        kmer.6.1 <- kmer.6[id1,1]
        kmer.6.2 <- kmer.6[id1,2]
        preferred.1 <- data.frame(six.mer=kmer.6.1,six.mer.rc=kmer.6.2,p.value=pval.1[id1],p.adjust=padj.1[id1],mean=mean.1[id1],diff=diff.1[id1])
    }
    
    padj.2 <- p.adjust(pval.2, method=p.adjust.method)
    k2 <- sum(padj.2<level)
    preferred.2 <- NULL
    if(k2 > 0)
    {
        id2 <- sort(pval.2,decreasing=F,index.return=T)$ix[1:k2]
        kmer.6.1 <- kmer.6[id2,1]
        kmer.6.2 <- kmer.6[id2,2]
        preferred.2 <- data.frame(six.mer=kmer.6.1,six.mer.rc=kmer.6.2,p.value=pval.2[id2],p.adjust=padj.2[id2],mean=mean.2[id2],diff=diff.2[id2])
    }
    
    return(list("preferred.1"=preferred.1,"preferred.2"=preferred.2,"file1"=file1,"file2"=file2,"mu1"=mu1,"mu2"=mu2,"level"=level,"p.adjust.method"=p.adjust.method))
}

#########################
##Plot Preferred 6-mers##
#########################
preferred.plot <- function(dir, file1, file2, kmers.6, xlab="Replicate 1",ylab = "Replicate 2",xlim=c(-0.5,0.5),ylim=c(-0.5,0.5),main="Scatter Plot of 8-mer E-scores")
{
    setwd(dir)
    
    data1 <- read.table(file1,header=F,sep="\t")
    x1 <- as.vector(data1$V3)
    x1 <- as.numeric(x1[2:length(x1)])
    kmers.f <- as.vector(data1$V1)
    kmers.f <- kmers.f[2:length(kmers.f)]
    kmers.r <- as.vector(data1$V2)
    kmers.r <- kmers.r[2:length(kmers.r)]
    kmers <- cbind(kmers.f,kmers.r)
    data2 <- read.table(file2,header=F,sep="\t")
    x2 <- as.vector(data2$V3)
    x2 <- as.numeric(x2[2:length(x2)])
    
    n <- length(kmers.6)
    if(n>0) {
        plot(x1,x2,col="grey",xlab=xlab,ylab=ylab,main=main,xlim=xlim,ylim=ylim,pch=19,cex=0.5)
        abline(h=0)
        abline(v=0)
        abline(a=0,b=1,lty=2)
        names <- NULL
        for(i in 1:n)
        {
            par(new=T)
            id1 <- grep(kmers.6[i],as.character(kmers[,1]))
            id2 <- grep(kmers.6[i],as.character(kmers[,2]))
            id <- unique(c(id1,id2))
            plot(x1[id],x2[id],col=i,xlab=xlab,ylab=ylab,xlim=xlim,ylim=ylim,pch=19,cex=0.8)
            names <- c(names,kmers.6[i])
        }
        legend(-0.5, 0.5, names, cex=0.8, col=1:n, pch=1)
    }
}