#########################################
##Specification of dir, file1 and file2##
#########################################

args <- commandArgs(trailingOnly = TRUE)

##Files have the first three columns as: "8-mer, 8-mer(reverse complementary), and observed E-score" and separated by tab
#1st PBM observed E-score file
file1 <- args[1]

#2nd PBM observed E-score file
file2 <- args[2]

##specify the set of 6-mers to be plotted, if NULL top k singificant 6-mers will be plotted
kmers.6 <- NULL
k <- as.numeric(args[3])

##Specifies the filename to save the plot to
out_file = args[4]

name1 <- args[5]
name2 <- args[6]

#Load R script 'preferred.analysis.r' and R data 'kmer.6.RData'
R.script.path <- args[7] 
source(paste(R.script.path,"preferred.analysis.r",sep=""))
load(paste(R.script.path, "kmer.6.RData", sep=""))

#Get current direcotry
dir <- getwd()

#########################
##Find Preferred 6-mers##
#########################
#caculat p-values and output significant 6-mers: preferred.pvalue(dir, file1, file2, mu1=2.0, mu2=0.0, level=0.05, p.adjust.method="fdr")
##mu1 and mu2 controls the practical significant level, mu2 may need to be adjusted to 0.5 if practical significance is too small
result <- preferred.pvalue(dir, mu2=0.0, file1, file2)
#6-mers preferred by the 1st pbm experiment
result$preferred.1
#6-mers preferred by the 2nd pbm experiment
result$preferred.2

#########################
##Plot Preferred 6-mers##
#########################
if(length(kmers.6)==0)
{
    #plot top k 6-mers if kmers.6 is not specified
    kmers.6.1 <- as.character(result$preferred.1$six.mer)
    kmers.6.2 <- as.character(result$preferred.2$six.mer)
    kmers.6 <- c(kmers.6.1[1:min(length(kmers.6.1),k)],kmers.6.2[1:min(length(kmers.6.2),k)])
}
#scatter plot: preferred.plot(dir, file1, file2, kmers.6, xlab="Replicate 1",ylab = "Replicate 2",xlim=c(-0.5,0.5),ylim=c(-0.5,0.5),main="Scatter Plot of 8-mer E-scores")
png(out_file ,width = 1000, height = 1000)
preferred.plot(dir, file1, file2, kmers.6,xlab=name1,ylab=name2)
dev.off()

###########################
##Output Preferred 6-mers##
###########################
setwd(dir)
write.table(result$preferred.1, sep="\t", file="preferred.tf1.txt",row.names=F,quote=F)
write.table(result$preferred.2, sep="\t", file="preferred.tf2.txt",row.names=F,quote=F)

